@@ -1,7 +1,7 @@
 /*  bam2depth.c -- depth subcommand.
 
     Copyright (C) 2011, 2012 Broad Institute.
-    Copyright (C) 2012-2014 Genome Research Ltd.
+    Copyright (C) 2012-2016, 2018, 2019 Genome Research Ltd.
 
     Author: Heng Li <lh3@sanger.ac.uk>
 
@@ -39,20 +39,19 @@ DEALINGS IN THE SOFTWARE.  */
 #include <unistd.h>
 #include "htslib/sam.h"
 #include "samtools.h"
+#include "bedidx.h"
 #include "sam_opts.h"
 
+#define BAM_FMAX ((BAM_FSUPPLEMENTARY << 1) - 1)
+
 typedef struct {     // auxiliary data structure
     samFile *fp;     // the file handle
-    bam_hdr_t *hdr;  // the file header
+    sam_hdr_t *hdr;  // the file header
     hts_itr_t *iter; // NULL if a region not specified
     int min_mapQ, min_len; // mapQ filter; length filter
+    uint32_t flags;  // read filtering flags
 } aux_t;
 
-void *bed_read(const char *fn); // read a BED or position list file
-void bed_destroy(void *_h);     // destroy the BED data structure
-int bed_overlap(const void *_h, const char *chr, int beg, int end); // test if chr:beg-end overlaps
-int bed_query(const void *_h, const char *chr, int pos, int *beg, int *end);
-
 // This function reads a BAM alignment from one BAM file.
 static int read_bam(void *data, bam1_t *b) // read level filters better go here to avoid pileup
 {
@@ -62,7 +61,7 @@ static int read_bam(void *data, bam1_t *b) // read level filters better go here
     {
         ret = aux->iter? sam_itr_next(aux->fp, aux->iter, b) : sam_read1(aux->fp, aux->hdr, b);
         if ( ret<0 ) break;
-        if ( b->core.flag & (BAM_FUNMAP | BAM_FSECONDARY | BAM_FQCFAIL | BAM_FDUP) ) continue;
+        if ( b->core.flag & aux->flags) continue;
         if ( (int)b->core.qual < aux->min_mapQ ) continue;
         if ( aux->min_len && bam_cigar2qlen(b->core.n_cigar, bam_get_cigar(b)) < aux->min_len ) continue;
         break;
@@ -79,14 +78,21 @@ static int usage() {
     fprintf(stderr, "   -a                  output all positions (including zero depth)\n");
     fprintf(stderr, "   -a -a (or -aa)      output absolutely all positions, including unused ref. sequences\n");
     fprintf(stderr, "   -b <bed>            list of positions or regions\n");
+    fprintf(stderr, "   -X                  use customized index files\n");
     fprintf(stderr, "   -f <list>           list of input BAM filenames, one per line [null]\n");
-    fprintf(stderr, "   -l <int>            read length threshold (ignore reads shorter than <int>)\n");
-    fprintf(stderr, "   -d/-m <int>         maximum coverage depth [8000]\n");  // the htslib's default
-    fprintf(stderr, "   -q <int>            base quality threshold\n");
-    fprintf(stderr, "   -Q <int>            mapping quality threshold\n");
+    fprintf(stderr, "   -H                  print a file header\n");
+    fprintf(stderr, "   -l <int>            read length threshold (ignore reads shorter than <int>) [0]\n");
+    fprintf(stderr, "   -d/-m <int>         maximum coverage depth [8000]. If 0, depth is set to the maximum\n"
+                    "                       integer value, effectively removing any depth limit.\n");  // the htslib's default
+    fprintf(stderr, "   -o FILE             where to write output to [stdout]\n");
+    fprintf(stderr, "   -q <int>            base quality threshold [0]\n");
+    fprintf(stderr, "   -Q <int>            mapping quality threshold [0]\n");
     fprintf(stderr, "   -r <chr:from-to>    region\n");
+    fprintf(stderr, "   -g <flags>          include reads that have any of the specified flags set [0]\n");
+    fprintf(stderr, "   -G <flags>          filter out reads that have any of the specified flags set"
+                    "                       [UNMAP,SECONDARY,QCFAIL,DUP]\n");
 
-    sam_global_opt_help(stderr, "-.--.");
+    sam_global_opt_help(stderr, "-.--.--.");
 
     fprintf(stderr, "\n");
     fprintf(stderr, "The output is a simple tab-separated table with three columns: reference name,\n");
@@ -94,42 +100,70 @@ static int usage() {
     fprintf(stderr, "omitted by default; see the -a option.\n");
     fprintf(stderr, "\n");
 
-    return 1;
+    return EXIT_FAILURE;
 }
 
 int main_depth(int argc, char *argv[])
 {
-    int i, n, tid, beg, end, pos, *n_plp, baseQ = 0, mapQ = 0, min_len = 0;
+    int i, n, tid, reg_tid, *n_plp, baseQ = 0, mapQ = 0, min_len = 0, has_index_file = 0;
+    hts_pos_t beg, end, pos, last_pos = -1;
     int all = 0, status = EXIT_SUCCESS, nfiles, max_depth = -1;
     const bam_pileup1_t **plp;
     char *reg = 0; // specified region
     void *bed = 0; // BED data structure
     char *file_list = NULL, **fn = NULL;
-    bam_hdr_t *h = NULL; // BAM header of the 1st input
+    sam_hdr_t *h = NULL; // BAM header of the 1st input
     aux_t **data;
     bam_mplp_t mplp;
-    int last_pos = -1, last_tid = -1, ret;
+    int last_tid = -1, ret;
+    int print_header = 0;
+    char *output_file = NULL;
+    FILE *file_out = stdout;
+    uint32_t flags = (BAM_FUNMAP | BAM_FSECONDARY | BAM_FQCFAIL | BAM_FDUP);
+    int tflags = 0;
 
     sam_global_args ga = SAM_GLOBAL_ARGS_INIT;
     static const struct option lopts[] = {
-        SAM_OPT_GLOBAL_OPTIONS('-', 0, '-', '-', 0),
+        SAM_OPT_GLOBAL_OPTIONS('-', 0, '-', '-', 0, '-'),
         { NULL, 0, NULL, 0 }
     };
 
     // parse the command line
-    while ((n = getopt_long(argc, argv, "r:b:q:Q:l:f:am:d:", lopts, NULL)) >= 0) {
+    while ((n = getopt_long(argc, argv, "r:b:Xq:Q:l:f:am:d:Ho:g:G:", lopts, NULL)) >= 0) {
         switch (n) {
             case 'l': min_len = atoi(optarg); break; // minimum query length
             case 'r': reg = strdup(optarg); break;   // parsing a region requires a BAM header
             case 'b':
                 bed = bed_read(optarg); // BED or position list file can be parsed now
-                if (!bed) { print_error_errno("depth", "Could not read file \"%s\"", optarg); return 1; }
+                if (!bed) {
+                    print_error_errno("depth", "Could not read file \"%s\"", optarg);
+                    return EXIT_FAILURE;
+                }
                 break;
+            case 'X': has_index_file = 1; break;
             case 'q': baseQ = atoi(optarg); break;   // base quality threshold
             case 'Q': mapQ = atoi(optarg); break;    // mapping quality threshold
             case 'f': file_list = optarg; break;
             case 'a': all++; break;
             case 'd': case 'm': max_depth = atoi(optarg); break; // maximum coverage depth
+            case 'H': print_header = 1; break;
+            case 'o': output_file = optarg; break;
+            case 'g':
+                tflags = bam_str2flag(optarg);
+                if (tflags < 0 || tflags > BAM_FMAX) {
+                    print_error_errno("depth", "Flag value \"%s\" is not supported", optarg);
+                    return 1;
+                }
+                flags &= ~tflags;
+                break;
+            case 'G':
+                tflags = bam_str2flag(optarg);
+                if (tflags < 0 || tflags > BAM_FMAX) {
+                    print_error_errno("depth", "Flag value \"%s\" is not supported", optarg);
+                    return 1;
+                }
+                flags |= tflags;
+                break;
             default:  if (parse_sam_global_opt(n, optarg, lopts, &ga) == 0) break;
                       /* else fall-through */
             case '?': return usage();
@@ -138,18 +172,40 @@ int main_depth(int argc, char *argv[])
     if (optind == argc && !file_list)
         return usage();
 
+    /* output file provided by user */
+    if (output_file != NULL && strcmp(output_file,"-")!=0) {
+        file_out = fopen( output_file, "w" );
+        if (file_out == NULL) {
+            print_error_errno("depth", "Cannot open \"%s\" for writing.", output_file);
+            return EXIT_FAILURE;
+        }
+    }
+
+
     // initialize the auxiliary data structures
     if (file_list)
     {
-        if ( read_file_list(file_list,&nfiles,&fn) ) return 1;
+        if (has_index_file) {
+            print_error("depth", "The -f option cannot be combined with -X");
+            return 1;
+        }
+        if ( read_file_list(file_list,&nfiles,&fn) ) return EXIT_FAILURE;
         n = nfiles;
         argv = fn;
         optind = 0;
     }
-    else
-        n = argc - optind; // the number of BAMs on the command line
+    else if (has_index_file) { // Calculate # of input BAM files
+        if ((argc - optind) % 2 != 0) {
+            fprintf(stderr, "Error: Odd number of filenames detected! Each BAM file should have an index file\n");
+            return 1;
+        }
+        n = (argc - optind) / 2;
+    } else {
+        n = argc - optind;
+    }
     data = calloc(n, sizeof(aux_t*)); // data[i] for the i-th input
-    beg = 0; end = INT_MAX;  // set the default region
+    reg_tid = 0; beg = 0; end = HTS_POS_MAX;  // set the default region
+
     for (i = 0; i < n; ++i) {
         int rf;
         data[i] = calloc(1, sizeof(aux_t));
@@ -162,24 +218,32 @@ int main_depth(int argc, char *argv[])
         rf = SAM_FLAG | SAM_RNAME | SAM_POS | SAM_MAPQ | SAM_CIGAR | SAM_SEQ;
         if (baseQ) rf |= SAM_QUAL;
         if (hts_set_opt(data[i]->fp, CRAM_OPT_REQUIRED_FIELDS, rf)) {
-            fprintf(stderr, "Failed to set CRAM_OPT_REQUIRED_FIELDS value\n");
-            return 1;
+            print_error_errno("depth", "Failed to set CRAM_OPT_REQUIRED_FIELDS value");
+            status = EXIT_FAILURE;
+            goto depth_end;
         }
         if (hts_set_opt(data[i]->fp, CRAM_OPT_DECODE_MD, 0)) {
-            fprintf(stderr, "Failed to set CRAM_OPT_DECODE_MD value\n");
-            return 1;
+            print_error_errno("depth", "Failed to set CRAM_OPT_DECODE_MD value");
+            status = EXIT_FAILURE;
+            goto depth_end;
         }
         data[i]->min_mapQ = mapQ;                    // set the mapQ filter
         data[i]->min_len  = min_len;                 // set the qlen filter
         data[i]->hdr = sam_hdr_read(data[i]->fp);    // read the BAM header
         if (data[i]->hdr == NULL) {
-            fprintf(stderr, "Couldn't read header for \"%s\"\n",
-                    argv[optind+i]);
+            print_error_errno("depth", "Couldn't read header for \"%s\"",
+                              argv[optind+i]);
             status = EXIT_FAILURE;
             goto depth_end;
         }
         if (reg) { // if a region is specified
-            hts_idx_t *idx = sam_index_load(data[i]->fp, argv[optind+i]);  // load the index
+            hts_idx_t *idx = NULL;
+            // If index filename has not been specfied, look in BAM folder
+            if (has_index_file) {
+                idx = sam_index_load2(data[i]->fp, argv[optind+i], argv[optind+i+n]);  // load the index
+            } else {
+                idx = sam_index_load(data[i]->fp, argv[optind+i]);
+            }
             if (idx == NULL) {
                 print_error("depth", "can't load index for \"%s\"", argv[optind+i]);
                 status = EXIT_FAILURE;
@@ -193,66 +257,84 @@ int main_depth(int argc, char *argv[])
                 goto depth_end;
             }
         }
+        data[i]->flags = flags;
     }
-
+    if (print_header) {
+        fputs("#CHROM\tPOS", file_out);
+        for (i = 0; i < n; ++i) {
+            fputc('\t', file_out);
+            fputs(argv[optind+i], file_out);
+            }
+        fputc('\n', file_out);
+        }
     h = data[0]->hdr; // easy access to the header of the 1st BAM
     if (reg) {
         beg = data[0]->iter->beg; // and to the parsed region coordinates
         end = data[0]->iter->end;
+        reg_tid = data[0]->iter->tid;
     }
 
     // the core multi-pileup loop
     mplp = bam_mplp_init(n, read_bam, (void**)data); // initialization
     if (0 < max_depth)
         bam_mplp_set_maxcnt(mplp,max_depth);  // set maximum coverage depth
+    else if (!max_depth)
+        bam_mplp_set_maxcnt(mplp,INT_MAX);
     n_plp = calloc(n, sizeof(int)); // n_plp[i] is the number of covering reads from the i-th BAM
     plp = calloc(n, sizeof(bam_pileup1_t*)); // plp[i] points to the array of covering reads (internal in mplp)
-    while ((ret=bam_mplp_auto(mplp, &tid, &pos, n_plp, plp)) > 0) { // come to the next covered position
+    while ((ret=bam_mplp64_auto(mplp, &tid, &pos, n_plp, plp)) > 0) { // come to the next covered position
         if (pos < beg || pos >= end) continue; // out of range; skip
-        if (tid >= h->n_targets) continue;     // diff number of @SQ lines per file?
-        if (bed && bed_overlap(bed, h->target_name[tid], pos, pos + 1) == 0) continue; // not in BED; skip
+        if (tid >= sam_hdr_nref(h)) continue;     // diff number of @SQ lines per file?
         if (all) {
             while (tid > last_tid) {
-                if (last_tid >= 0 && all > 1 && !reg) {
-                    // Deal with remainder or entirety of last tid
-                    while (++last_pos < h->target_len[last_tid]) {
-                        if (bed && bed_overlap(bed, h->target_name[last_tid], last_pos, last_pos + 1) == 0)
+                if (last_tid >= 0 && !reg) {
+                    // Deal with remainder or entirety of last tid.
+                    while (++last_pos < sam_hdr_tid2len(h, last_tid)) {
+                        // Horribly inefficient, but the bed API is an obfuscated black box.
+                        if (bed && bed_overlap(bed, sam_hdr_tid2name(h, last_tid), last_pos, last_pos + 1) == 0)
                             continue;
-                        fputs(h->target_name[last_tid], stdout); printf("\t%d", last_pos+1);
+                        fputs(sam_hdr_tid2name(h, last_tid), file_out);
+                        fprintf(file_out, "\t%"PRIhts_pos, last_pos+1);
                         for (i = 0; i < n; i++)
-                            putchar('\t'), putchar('0');
-                        putchar('\n');
+                            fputc('\t', file_out), fputc('0', file_out);
+                        fputc('\n', file_out);
                     }
                 }
                 last_tid++;
                 last_pos = -1;
+                if (all < 2)
+                    break;
             }
 
             // Deal with missing portion of current tid
             while (++last_pos < pos) {
                 if (last_pos < beg) continue; // out of range; skip
-                if (bed && bed_overlap(bed, h->target_name[tid], last_pos, last_pos + 1) == 0)
+                if (bed && bed_overlap(bed, sam_hdr_tid2name(h, tid), last_pos, last_pos + 1) == 0)
                     continue;
-                fputs(h->target_name[tid], stdout); printf("\t%d", last_pos+1);
+                fputs(sam_hdr_tid2name(h, tid), file_out);
+                fprintf(file_out, "\t%"PRIhts_pos, last_pos+1);
                 for (i = 0; i < n; i++)
-                    putchar('\t'), putchar('0');
-                putchar('\n');
+                    fputc('\t', file_out), fputc('0', file_out);
+                fputc('\n', file_out);
             }
 
             last_tid = tid;
             last_pos = pos;
         }
-        fputs(h->target_name[tid], stdout); printf("\t%d", pos+1); // a customized printf() would be faster
+        if (bed && bed_overlap(bed, sam_hdr_tid2name(h, tid), pos, pos + 1) == 0) continue;
+        fputs(sam_hdr_tid2name(h, tid), file_out);
+        fprintf(file_out, "\t%"PRIhts_pos, pos+1); // a customized printf() would be faster
         for (i = 0; i < n; ++i) { // base level filters have to go here
             int j, m = 0;
             for (j = 0; j < n_plp[i]; ++j) {
                 const bam_pileup1_t *p = plp[i] + j; // DON'T modfity plp[][] unless you really know
                 if (p->is_del || p->is_refskip) ++m; // having dels or refskips at tid:pos
-                else if (bam_get_qual(p->b)[p->qpos] < baseQ) ++m; // low base quality
+                else if (p->qpos < p->b->core.l_qseq &&
+                         bam_get_qual(p->b)[p->qpos] < baseQ) ++m; // low base quality
             }
-            printf("\t%d", n_plp[i] - m); // this the depth to output
+            fprintf(file_out, "\t%d", n_plp[i] - m); // this the depth to output
         }
-        putchar('\n');
+        fputc('\n', file_out);
     }
     if (ret < 0) status = EXIT_FAILURE;
     free(n_plp); free(plp);
@@ -260,15 +342,20 @@ int main_depth(int argc, char *argv[])
 
     if (all) {
         // Handle terminating region
-        while (last_tid < h->n_targets) {
-            while (++last_pos < h->target_len[last_tid]) {
+        if (last_tid < 0 && reg) {
+            last_tid = reg_tid;
+            last_pos = beg-1;
+        }
+        while (last_tid >= 0 && last_tid < sam_hdr_nref(h)) {
+            while (++last_pos < sam_hdr_tid2len(h, last_tid)) {
                 if (last_pos >= end) break;
-                if (bed && bed_overlap(bed, h->target_name[last_tid], last_pos, last_pos + 1) == 0)
+                if (bed && bed_overlap(bed, sam_hdr_tid2name(h, last_tid), last_pos, last_pos + 1) == 0)
                     continue;
-                fputs(h->target_name[last_tid], stdout); printf("\t%d", last_pos+1);
+                fputs(sam_hdr_tid2name(h, last_tid), file_out);
+                fprintf(file_out, "\t%"PRIhts_pos, last_pos+1);
                 for (i = 0; i < n; i++)
-                    putchar('\t'), putchar('0');
-                putchar('\n');
+                    fputc('\t', file_out), fputc('0', file_out);
+                fputc('\n', file_out);
             }
             last_tid++;
             last_pos = -1;
@@ -278,8 +365,17 @@ int main_depth(int argc, char *argv[])
     }
 
 depth_end:
+    if (fclose(file_out) != 0) {
+        if (status == EXIT_SUCCESS) {
+            print_error_errno("depth", "error on closing \"%s\"",
+                              (output_file && strcmp(output_file, "-") != 0
+                               ? output_file : "stdout"));
+            status = EXIT_FAILURE;
+        }
+    }
+
     for (i = 0; i < n && data[i]; ++i) {
-        bam_hdr_destroy(data[i]->hdr);
+        sam_hdr_destroy(data[i]->hdr);
         if (data[i]->fp) sam_close(data[i]->fp);
         hts_itr_destroy(data[i]->iter);
         free(data[i]);
