@@ -1,6 +1,6 @@
 /*  bam_split.c -- split subcommand.
 
-    Copyright (C) 2013-2015 Genome Research Ltd.
+    Copyright (C) 2013-2016,2018-2019 Genome Research Ltd.
 
     Author: Martin Pollard <mp15@sanger.ac.uk>
 
@@ -24,7 +24,6 @@ DEALINGS IN THE SOFTWARE.  */
 
 #include <config.h>
 
-#include <htslib/sam.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -32,19 +31,25 @@ DEALINGS IN THE SOFTWARE.  */
 #include <limits.h>
 #include <unistd.h>
 #include <regex.h>
+#include <assert.h>
+#include <htslib/sam.h>
 #include <htslib/khash.h>
 #include <htslib/kstring.h>
+#include <htslib/cram.h>
+#include "htslib/thread_pool.h"
 #include "sam_opts.h"
+#include "samtools.h"
 
 
 KHASH_MAP_INIT_STR(c2i, int)
 
 struct parsed_opts {
-    char* merged_input_name;
-    char* unaccounted_header_name;
-    char* unaccounted_name;
-    char* output_format_string;
+    const char *merged_input_name;
+    const char *unaccounted_header_name;
+    const char *unaccounted_name;
+    const char *output_format_string;
     bool verbose;
+    int no_pg;
     sam_global_args ga;
 };
 
@@ -52,15 +57,18 @@ typedef struct parsed_opts parsed_opts_t;
 
 struct state {
     samFile* merged_input_file;
-    bam_hdr_t* merged_input_header;
+    sam_hdr_t* merged_input_header;
     samFile* unaccounted_file;
-    bam_hdr_t* unaccounted_header;
+    sam_hdr_t* unaccounted_header;
     size_t output_count;
     char** rg_id;
+    char **rg_index_file_name;
     char **rg_output_file_name;
     samFile** rg_output_file;
-    bam_hdr_t** rg_output_header;
+    sam_hdr_t** rg_output_header;
     kh_c2i_t* rg_hash;
+    htsThreadPool p;
+    int write_index;
 };
 
 typedef struct state state_t;
@@ -71,14 +79,15 @@ static void cleanup_opts(parsed_opts_t* opts);
 static void usage(FILE *write_to)
 {
     fprintf(write_to,
-"Usage: samtools split [-u <unaccounted.bam>[:<unaccounted_header.sam>]]\n"
+"Usage: samtools split [-u <unaccounted.bam>] [-h <unaccounted_header.sam>]\n"
 "                      [-f <format_string>] [-v] <merged.bam>\n"
 "Options:\n"
 "  -f STRING       output filename format string [\"%%*_%%#.%%.\"]\n"
 "  -u FILE1        put reads with no RG tag or an unrecognised RG tag in FILE1\n"
-"  -u FILE1:FILE2  ...and override the header with FILE2\n"
-"  -v              verbose output\n");
-    sam_global_opt_help(write_to, "-....");
+"  -h FILE2        ... and override the header with FILE2 (-u file only)\n"
+"  -v              verbose output\n"
+"  --no-PG         do not add a PG line\n");
+    sam_global_opt_help(write_to, "-....@..");
     fprintf(write_to,
 "\n"
 "Format string expansions:\n"
@@ -95,11 +104,11 @@ static parsed_opts_t* parse_args(int argc, char** argv)
 {
     if (argc == 1) { usage(stdout); return NULL; }
 
-    const char* optstring = "vf:u:";
-    char* delim;
+    const char *optstring = "vf:h:u:@:";
 
     static const struct option lopts[] = {
-        SAM_OPT_GLOBAL_OPTIONS('-', 0, 0, 0, 0),
+        SAM_OPT_GLOBAL_OPTIONS('-', 0, 0, 0, 0, '@'),
+        {"no-PG", no_argument, NULL, 1},
         { NULL, 0, NULL, 0 }
     };
 
@@ -112,20 +121,19 @@ static parsed_opts_t* parse_args(int argc, char** argv)
     while ((opt = getopt_long(argc, argv, optstring, lopts, NULL)) != -1) {
         switch (opt) {
         case 'f':
-            retval->output_format_string = strdup(optarg);
-            if (! retval->output_format_string ) { perror("cannot allocate output format string memory"); return NULL; }
+            retval->output_format_string = optarg;
+            break;
+        case 'h':
+            retval->unaccounted_header_name = optarg;
             break;
         case 'v':
             retval->verbose = true;
             break;
         case 'u':
-            retval->unaccounted_name = strdup(optarg);
-            if (! retval->unaccounted_name ) { perror("cannot allocate string memory"); return NULL; }
-            if ((delim = strchr(retval->unaccounted_name, ':')) != NULL) {
-                *delim = '\0';
-                retval->unaccounted_header_name = strdup(delim+1);
-                if (! retval->unaccounted_header_name ) { perror("cannot allocate string memory"); return NULL; }
-            }
+            retval->unaccounted_name = optarg;
+            break;
+        case 1:
+            retval->no_pg = 1;
             break;
         default:
             if (parse_sam_global_opt(opt, optarg, lopts, &retval->ga) == 0) break;
@@ -137,20 +145,19 @@ static parsed_opts_t* parse_args(int argc, char** argv)
         }
     }
 
-    if (retval->output_format_string == NULL) retval->output_format_string = strdup("%*_%#.%.");
+    if (retval->output_format_string == NULL) retval->output_format_string = "%*_%#.%.";
 
     argc -= optind;
     argv += optind;
 
     if (argc != 1) {
-        fprintf(stderr, "Invalid number of arguments: %d\n", argc);
+        print_error("split", "Invalid number of arguments: %d", argc);
         usage(stderr);
         free(retval);
         return NULL;
     }
 
-    retval->merged_input_name = strdup(argv[0]);
-    if (! retval->merged_input_name ) { perror("cannot allocate string memory"); return NULL; }
+    retval->merged_input_name = argv[0];
 
     return retval;
 }
@@ -162,181 +169,140 @@ static char* expand_format_string(const char* format_string, const char* basenam
     const char* pointer = format_string;
     const char* next;
     while ((next = strchr(pointer, '%')) != NULL) {
-        kputsn(pointer, next-pointer, &str);
+        if (kputsn(pointer, next-pointer, &str) < 0) goto memfail;
         ++next;
         switch (*next) {
             case '%':
-                kputc('%', &str);
+                if (kputc('%', &str) < 0) goto memfail;
                 break;
             case '*':
-                kputs(basename, &str);
+                if (kputs(basename, &str) < 0) goto memfail;
                 break;
             case '#':
-                kputl(rg_idx, &str);
+                if (kputl(rg_idx, &str) < 0) goto memfail;
                 break;
             case '!':
-                kputs(rg_id, &str);
+                if (kputs(rg_id, &str) < 0) goto memfail;
                 break;
             case '.':
                 // Only really need to cope with sam, bam, cram
-                if (format->format != unknown_format)
-                    kputs(hts_format_file_extension(format), &str);
-                else
-                    kputs("bam", &str);
+                if (format->format != unknown_format) {
+                    if (kputs(hts_format_file_extension(format), &str) < 0)
+                        goto memfail;
+                } else {
+                    if (kputs("bam", &str) < 0) goto memfail;
+                }
                 break;
             case '\0':
-                // Error is: fprintf(stderr, "bad format string, trailing %%\n");
-                free(str.s);
-                return NULL;
+                print_error("split", "Trailing %% in filename format string");
+                goto fail;
             default:
                 // Error is: fprintf(stderr, "bad format string, unknown format specifier\n");
-                free(str.s);
-                return NULL;
+                print_error("split", "Unknown specifier %%%c in filename format string", *next);
+                goto fail;
         }
         pointer = next + 1;
     }
-    kputs(pointer, &str);
+    if (kputs(pointer, &str) < 0) goto memfail;
     return ks_release(&str);
+
+ memfail:
+    print_error_errno("split", "Couldn't build output filename");
+ fail:
+    free(str.s);
+    return NULL;
 }
 
 // Parse the header, count the number of RG tags and return a list of their names
-static bool count_RG(bam_hdr_t* hdr, size_t* count, char*** output_name)
+static bool count_RG(sam_hdr_t* hdr, size_t* count, char*** output_name)
 {
-    if (hdr->l_text < 3 ) {
+    char **names = NULL;
+    kstring_t id_val = KS_INITIALIZE;
+    int i, n_rg = sam_hdr_count_lines(hdr, "RG");
+
+    if (n_rg < 0) {
+        print_error("split", "Failed to get @RG IDs");
+        *count = 0;
+        *output_name = NULL;
+        return false;
+    }
+
+    if (n_rg == 0) {
         *count = 0;
         *output_name = NULL;
         return true;
     }
-    kstring_t input = { 0, 0, NULL };
-    kputsn(hdr->text, hdr->l_text, &input);
 
-    //////////////////////////////////////////
-    // First stage count number of @RG tags //
-    //////////////////////////////////////////
-    char* pointer = ks_str(&input);
-    size_t n_rg = 0;
-    // Guard against rare case where @RG is first header line
-    // This shouldn't happen but could where @HD is omitted
-    if (pointer[0] == '@' && pointer[1] == 'R' && pointer[2] == 'G' ) {
-        ++n_rg;
-        pointer += 3;
-    }
-    char* line;
-    while ((line = strstr(pointer, "\n@RG")) != NULL) {
-        ++n_rg;
-        pointer = line + 1;
+    names = calloc(n_rg, sizeof(names[0]));
+    if (!names) goto memfail;
+
+    for (i = 0; i < n_rg; i++) {
+        if (sam_hdr_find_tag_pos(hdr, "RG", i, "ID", &id_val) < 0) goto memfail;
+        names[i] = ks_release(&id_val);
     }
 
-    //////////////////////////////////
-    // Second stage locate @RG ID's //
-    //////////////////////////////////
-    char** names = (char**)calloc(sizeof(char*), n_rg);
-    size_t next = 0;
-
-    regex_t rg_finder;
-    if (regcomp(&rg_finder, "^@RG.*\tID:([!-)+-<>-~][ !-~]*)(\t.*$|$)", REG_EXTENDED|REG_NEWLINE) != 0) {
-        free(input.s);
-        free(names);
-        return false;
-    }
-    regmatch_t* matches = (regmatch_t*)calloc(sizeof(regmatch_t),2);
-    int error;
-    char* begin = ks_str(&input);
-
-    while ((error = regexec(&rg_finder, begin, 2, matches, 0)) == 0) {
-        kstring_t str = { 0, 0, NULL };
-        kputsn(begin+matches[1].rm_so, matches[1].rm_eo-matches[1].rm_so, &str);
-        names[next++] = ks_release(&str);
-        begin += matches[0].rm_eo;
-    }
-
-    if (error != REG_NOMATCH) {
-        // cleanup
-        regfree(&rg_finder);
-        free(matches);
-        free(names);
-        free(input.s);
-        return false;
-    }
-    free(matches);
-
-    // return results
     *count = n_rg;
     *output_name = names;
-    regfree(&rg_finder);
-    free(input.s);
     return true;
+
+ memfail:
+    print_error_errno("split", "Failed to get @RG IDs");
+    *count = 0;
+    *output_name = NULL;
+    ks_free(&id_val);
+    free(names);
+    return false;
 }
 
-// Filters a header of @RG lines where ID != id_keep
-// TODO: strip @PG's descended from other RGs and their descendants
-static bool filter_header_rg(bam_hdr_t* hdr, const char* id_keep)
+static int header_compatible(sam_hdr_t *hdr1, sam_hdr_t *hdr2)
 {
-    kstring_t str = {0, 0, NULL};
-
-    regex_t rg_finder;
-
-    if (regcomp(&rg_finder, "^@RG.*\tID:([!-)+-<>-~][ !-~]*)(\t.*$|$)", REG_EXTENDED|REG_NEWLINE) != 0) {
-        return false;
+    size_t n;
+    if (sam_hdr_nref(hdr1) != sam_hdr_nref(hdr2)) {
+        print_error("split",
+                    "Unaccounted header contains wrong number of references");
+        return -1;
     }
-
-    // regex vars
-    char* header = hdr->text;
-    regmatch_t* matches = (regmatch_t*)calloc(sizeof(regmatch_t),2);
-    kstring_t found_id = { 0, 0, NULL };
-    int error;
-
-    while ((error = regexec(&rg_finder, header, 2, matches, 0)) == 0) {
-        kputsn(header, matches[0].rm_so, &str); // copy header up until the found RG line
-
-        found_id.l = 0;
-        kputsn(header+matches[1].rm_so, matches[1].rm_eo-matches[1].rm_so, &found_id); // extract ID
-        // if it matches keep keep it, else we can just ignore it
-        if (strcmp(ks_str(&found_id), id_keep) == 0) {
-            kputsn(header+matches[0].rm_so, (matches[0].rm_eo+1)-matches[0].rm_so, &str);
+    for (n = 0; n < sam_hdr_nref(hdr1); n++) {
+        hts_pos_t h1_len = sam_hdr_tid2len(hdr1, n);
+        hts_pos_t h2_len = sam_hdr_tid2len(hdr2, n);
+        if (h1_len != h2_len) {
+            print_error("split",
+                        "Unaccounted header reference %zu \"%s\" is not the same length as in the input file",
+                        n + 1, sam_hdr_tid2name(hdr2, n));
+            return -1;
         }
-        // move pointer forward
-        header += matches[0].rm_eo+1;
     }
-    // cleanup
-    free(found_id.s);
-    free(matches);
-    regfree(&rg_finder);
-    // Did we leave loop because of an error?
-    if (error != REG_NOMATCH) {
-        return false;
-    }
-
-    // Write remainder of string
-    kputs(header, &str);
-
-    // Modify header
-    hdr->l_text = ks_len(&str);
-    free(hdr->text);
-    hdr->text = ks_release(&str);
-
-    return true;
+    return 0;
 }
 
 // Set the initial state
-static state_t* init(parsed_opts_t* opts)
+static state_t* init(parsed_opts_t* opts, const char *arg_list)
 {
     state_t* retval = calloc(sizeof(state_t), 1);
     if (!retval) {
-        fprintf(stderr, "Out of memory");
+        print_error_errno("split", "Initialisation failed");
         return NULL;
     }
 
+    if (opts->ga.nthreads > 0) {
+        if (!(retval->p.pool = hts_tpool_init(opts->ga.nthreads))) {
+            fprintf(stderr, "Error creating thread pool\n");
+            cleanup_state(retval, false);
+            return NULL;
+        }
+    }
+
     retval->merged_input_file = sam_open_format(opts->merged_input_name, "rb", &opts->ga.in);
     if (!retval->merged_input_file) {
-        fprintf(stderr, "Could not open input file (%s)\n", opts->merged_input_name);
-        free(retval);
+        print_error_errno("split", "Could not open \"%s\"", opts->merged_input_name);
+        cleanup_state(retval, false);
         return NULL;
     }
+    if (retval->p.pool)
+        hts_set_opt(retval->merged_input_file, HTS_OPT_THREAD_POOL, &retval->p);
     retval->merged_input_header = sam_hdr_read(retval->merged_input_file);
     if (retval->merged_input_header == NULL) {
-        fprintf(stderr, "Could not read header for file '%s'\n",
-                opts->merged_input_name);
+        print_error("split", "Could not read header from \"%s\"", opts->merged_input_name);
         cleanup_state(retval, false);
         return NULL;
     }
@@ -345,40 +311,59 @@ static state_t* init(parsed_opts_t* opts)
         if (opts->unaccounted_header_name) {
             samFile* hdr_load = sam_open_format(opts->unaccounted_header_name, "r", &opts->ga.in);
             if (!hdr_load) {
-                fprintf(stderr, "Could not open unaccounted header file (%s)\n", opts->unaccounted_header_name);
+                print_error_errno("split", "Could not open unaccounted header file \"%s\"", opts->unaccounted_header_name);
                 cleanup_state(retval, false);
                 return NULL;
             }
             retval->unaccounted_header = sam_hdr_read(hdr_load);
             if (retval->unaccounted_header == NULL) {
-                fprintf(stderr, "Could not read header for file '%s'\n",
-                        opts->unaccounted_header_name);
+                print_error("split", "Could not read header from \"%s\"", opts->unaccounted_header_name);
                 cleanup_state(retval, false);
+                sam_close(hdr_load);
                 return NULL;
             }
             sam_close(hdr_load);
+            if (header_compatible(retval->merged_input_header,
+                                  retval->unaccounted_header) != 0) {
+                cleanup_state(retval, false);
+                return NULL;
+            }
         } else {
-            retval->unaccounted_header = bam_hdr_dup(retval->merged_input_header);
+            retval->unaccounted_header = sam_hdr_dup(retval->merged_input_header);
+            if (!opts->no_pg && sam_hdr_add_pg(retval->unaccounted_header, "samtools",
+                                    "VN", samtools_version(),
+                                    arg_list ? "CL": NULL,
+                                    arg_list ? arg_list : NULL,
+                                    NULL)) {
+                print_error("split", "Could not rewrite header for \"%s\"", opts->unaccounted_name);
+                cleanup_state(retval, false);
+                return NULL;
+            }
         }
 
         retval->unaccounted_file = sam_open_format(opts->unaccounted_name, "wb", &opts->ga.out);
         if (retval->unaccounted_file == NULL) {
-            fprintf(stderr, "Could not open unaccounted output file: %s\n", opts->unaccounted_name);
+            print_error_errno("split", "Could not open unaccounted output file \"%s\"", opts->unaccounted_name);
             cleanup_state(retval, false);
             return NULL;
         }
+        if (retval->p.pool)
+            hts_set_opt(retval->unaccounted_file, HTS_OPT_THREAD_POOL, &retval->p);
     }
 
     // Open output files for RGs
     if (!count_RG(retval->merged_input_header, &retval->output_count, &retval->rg_id)) return NULL;
     if (opts->verbose) fprintf(stderr, "@RG's found %zu\n",retval->output_count);
-
-    retval->rg_output_file_name = (char **)calloc(retval->output_count, sizeof(char *));
-    retval->rg_output_file = (samFile**)calloc(retval->output_count, sizeof(samFile*));
-    retval->rg_output_header = (bam_hdr_t**)calloc(retval->output_count, sizeof(bam_hdr_t*));
+    // Prevent calloc(0, size);
+    size_t num = retval->output_count ? retval->output_count : 1;
+    retval->rg_index_file_name = (char **)calloc(num, sizeof(char *));
+    retval->rg_output_file_name = (char **)calloc(num, sizeof(char *));
+    retval->rg_output_file = (samFile**)calloc(num, sizeof(samFile*));
+    retval->rg_output_header = (sam_hdr_t**)calloc(num, sizeof(sam_hdr_t*));
     retval->rg_hash = kh_init_c2i();
-    if (!retval->rg_output_file_name || !retval->rg_output_file || !retval->rg_output_header || !retval->rg_hash) {
-        fprintf(stderr, "Could not allocate memory for output file array. Out of memory?");
+    if (!retval->rg_output_file_name || !retval->rg_output_file || !retval->rg_output_header ||
+        !retval->rg_hash || !retval->rg_index_file_name) {
+        print_error_errno("split", "Could not initialise output file array");
         cleanup_state(retval, false);
         return NULL;
     }
@@ -386,7 +371,7 @@ static state_t* init(parsed_opts_t* opts)
     char* dirsep = strrchr(opts->merged_input_name, '/');
     char* input_base_name = strdup(dirsep? dirsep+1 : opts->merged_input_name);
     if (!input_base_name) {
-        fprintf(stderr, "Out of memory\n");
+        print_error_errno("split", "Filename manipulation failed");
         cleanup_state(retval, false);
         return NULL;
     }
@@ -403,7 +388,6 @@ static state_t* init(parsed_opts_t* opts)
                                                &opts->ga.out);
 
         if ( output_filename == NULL ) {
-            fprintf(stderr, "Error expanding output filename format string.\n");
             cleanup_state(retval, false);
             free(input_base_name);
             return NULL;
@@ -412,21 +396,35 @@ static state_t* init(parsed_opts_t* opts)
         retval->rg_output_file_name[i] = output_filename;
         retval->rg_output_file[i] = sam_open_format(output_filename, "wb", &opts->ga.out);
         if (retval->rg_output_file[i] == NULL) {
-            fprintf(stderr, "Could not open output file: %s\n", output_filename);
+            print_error_errno("split", "Could not open \"%s\"", output_filename);
             cleanup_state(retval, false);
             free(input_base_name);
             return NULL;
         }
+        if (retval->p.pool)
+            hts_set_opt(retval->rg_output_file[i], HTS_OPT_THREAD_POOL, &retval->p);
 
         // Record index in hash
         int ret;
         khiter_t iter = kh_put_c2i(retval->rg_hash, retval->rg_id[i], &ret);
+        if (ret < 0) {
+            print_error_errno("split", "Couldn't add @RG ID to look-up table");
+            cleanup_state(retval, false);
+            free(input_base_name);
+            return NULL;
+        }
         kh_val(retval->rg_hash,iter) = i;
 
         // Set and edit header
-        retval->rg_output_header[i] = bam_hdr_dup(retval->merged_input_header);
-        if ( !filter_header_rg(retval->rg_output_header[i], retval->rg_id[i]) ) {
-            fprintf(stderr, "Could not rewrite header for file: %s\n", output_filename);
+        retval->rg_output_header[i] = sam_hdr_dup(retval->merged_input_header);
+        if (sam_hdr_remove_except(retval->rg_output_header[i], "RG", "ID", retval->rg_id[i]) ||
+           (!opts->no_pg &&
+            sam_hdr_add_pg(retval->rg_output_header[i], "samtools",
+                        "VN", samtools_version(),
+                        arg_list ? "CL": NULL,
+                        arg_list ? arg_list : NULL,
+                        NULL))) {
+            print_error("split", "Could not rewrite header for \"%s\"", output_filename);
             cleanup_state(retval, false);
             free(input_base_name);
             return NULL;
@@ -434,6 +432,7 @@ static state_t* init(parsed_opts_t* opts)
     }
 
     free(input_base_name);
+    retval->write_index = opts->ga.write_index;
 
     return retval;
 }
@@ -441,16 +440,24 @@ static state_t* init(parsed_opts_t* opts)
 static bool split(state_t* state)
 {
     if (state->unaccounted_file && sam_hdr_write(state->unaccounted_file, state->unaccounted_header) != 0) {
-        fprintf(stderr, "Could not write output file header\n");
+        print_error_errno("split", "Could not write output file header");
         return false;
     }
     size_t i;
     for (i = 0; i < state->output_count; i++) {
         if (sam_hdr_write(state->rg_output_file[i], state->rg_output_header[i]) != 0) {
-            fprintf(stderr, "Could not write output file header for '%s'\n",
-                    state->rg_output_file_name[i]);
+            print_error_errno("split", "Could not write file header to \"%s\"", state->rg_output_file_name[i]);
             return false;
         }
+        if (state->write_index) {
+            state->rg_index_file_name[i] = auto_index(state->rg_output_file[i],
+                                                      state->rg_output_file_name[i],
+                                                      state->rg_output_header[i]);
+            if (!state->rg_index_file_name[i]) {
+                print_error_errno("split", "Could not create index for file \"%s\"", state->rg_output_file_name[i]);
+                return false;
+            }
+        }
     }
 
     bam1_t* file_read = bam_init1();
@@ -461,7 +468,7 @@ static bool split(state_t* state)
         bam_destroy1(file_read);
         file_read = NULL;
         if (r < -1) {
-            fprintf(stderr, "Could not read first input record\n");
+            print_error("split", "Could not read first input record");
             return false;
         }
     }
@@ -482,8 +489,7 @@ static bool split(state_t* state)
             // if found write to the appropriate untangled bam
             int i = kh_val(state->rg_hash,iter);
             if (sam_write1(state->rg_output_file[i], state->rg_output_header[i], file_read) < 0) {
-                fprintf(stderr, "Could not write to output file '%s'\n",
-                        state->rg_output_file_name[i]);
+                print_error_errno("split", "Could not write to \"%s\"", state->rg_output_file_name[i]);
                 bam_destroy1(file_read);
                 return false;
             }
@@ -499,7 +505,7 @@ static bool split(state_t* state)
                 return false;
             } else {
                 if (sam_write1(state->unaccounted_file, state->unaccounted_header, file_read) < 0) {
-                    fprintf(stderr, "Could not write to unaccounted output file\n");
+                    print_error_errno("split", "Could not write to unaccounted output file");
                     bam_destroy1(file_read);
                     return false;
                 }
@@ -512,12 +518,22 @@ static bool split(state_t* state)
             bam_destroy1(file_read);
             file_read = NULL;
             if (r < -1) {
-                fprintf(stderr, "Could not read input record\n");
+                print_error("split", "Could not read input record");
                 return false;
             }
         }
     }
 
+    if (state->write_index) {
+        for (i = 0; i < state->output_count; i++) {
+            if (sam_idx_save(state->rg_output_file[i]) < 0) {
+                print_error_errno("split", "writing index failed");
+                return false;
+            }
+            free(state->rg_index_file_name[i]);
+        }
+    }
+
     return true;
 }
 
@@ -526,10 +542,10 @@ static int cleanup_state(state_t* status, bool check_close)
     int ret = 0;
 
     if (!status) return 0;
-    if (status->unaccounted_header) bam_hdr_destroy(status->unaccounted_header);
+    if (status->unaccounted_header) sam_hdr_destroy(status->unaccounted_header);
     if (status->unaccounted_file) {
         if (sam_close(status->unaccounted_file) < 0 && check_close) {
-            fprintf(stderr, "Error on closing unaccounted file\n");
+            print_error("split", "Error on closing unaccounted file");
             ret = -1;
         }
     }
@@ -537,11 +553,10 @@ static int cleanup_state(state_t* status, bool check_close)
     size_t i;
     for (i = 0; i < status->output_count; i++) {
         if (status->rg_output_header && status->rg_output_header[i])
-            bam_hdr_destroy(status->rg_output_header[i]);
+            sam_hdr_destroy(status->rg_output_header[i]);
         if (status->rg_output_file && status->rg_output_file[i]) {
             if (sam_close(status->rg_output_file[i]) < 0 && check_close) {
-                fprintf(stderr, "Error on closing output file '%s'\n",
-                        status->rg_output_file_name[i]);
+                print_error("split", "Error on closing output file \"%s\"", status->rg_output_file_name[i]);
                 ret = -1;
             }
         }
@@ -549,12 +564,15 @@ static int cleanup_state(state_t* status, bool check_close)
         if (status->rg_output_file_name) free(status->rg_output_file_name[i]);
     }
     if (status->merged_input_header)
-        bam_hdr_destroy(status->merged_input_header);
+        sam_hdr_destroy(status->merged_input_header);
     free(status->rg_output_header);
     free(status->rg_output_file);
     free(status->rg_output_file_name);
+    free(status->rg_index_file_name);
     kh_destroy_c2i(status->rg_hash);
     free(status->rg_id);
+    if (status->p.pool)
+        hts_tpool_destroy(status->p.pool);
     free(status);
 
     return ret;
@@ -563,10 +581,6 @@ static int cleanup_state(state_t* status, bool check_close)
 static void cleanup_opts(parsed_opts_t* opts)
 {
     if (!opts) return;
-    free(opts->merged_input_name);
-    free(opts->unaccounted_header_name);
-    free(opts->unaccounted_name);
-    free(opts->output_format_string);
     sam_global_args_free(&opts->ga);
     free(opts);
 }
@@ -574,9 +588,12 @@ static void cleanup_opts(parsed_opts_t* opts)
 int main_split(int argc, char** argv)
 {
     int ret = 1;
+    char *arg_list = NULL;
     parsed_opts_t* opts = parse_args(argc, argv);
     if (!opts) goto cleanup_opts;
-    state_t* status = init(opts);
+    if (!opts->no_pg && !(arg_list = stringify_argv(argc+1, argv-1)))
+        goto cleanup_opts;
+    state_t* status = init(opts, arg_list);
     if (!status) goto cleanup_opts;
 
     if (!split(status)) {
@@ -588,6 +605,7 @@ int main_split(int argc, char** argv)
 
 cleanup_opts:
     cleanup_opts(opts);
+    free(arg_list);
 
     return ret;
 }
