@@ -2,7 +2,7 @@
 
 # The MIT License
 
-# Copyright (c) 2014, 2020 Genome Research Ltd.
+# Copyright (c) 2014 Genome Research Ltd.
 # Author: Rob Davies <rmd+sam@sanger.ac.uk>
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -95,11 +95,6 @@ if ($find) {
     # Otherwise read from STDIN
     process_file('STDIN', \*STDIN, $root_dir, $dest_re, $max_acc);
 }
-
-print "\n";
-print "Use environment REF_CACHE=$root_dir" . "/%2s" x $subdirs .
-    "/%s for accessing these files.\n";
-print "See also https://www.htslib.org/workflow/cram#the-ref_path-and-ref_cache for\nfurther information.\n";
 exit;
 
 sub find_files {
@@ -302,15 +297,15 @@ of the sequence.
 
 By default, sequences are stored in a hierarchy two directories deep, to
 keep the number of items in a single directory to a reasonable number.  This
-depth can be changed using the -subdirs option.
+depth can be chaged using the -subdirs option.
 
 If the -find option is used, the program will scan the given directory tree.
 Any files that appear to be fasta (by looking for a first line starting with
-'>' followed by something that looks like DNA sequence) will be read and
+'>' followed by somthing that looks like DNA sequence) will be read and
 added to the reference cache.  The traversal will ignore symbolic links.
 
 Samtools/htslib can be made to use the cache by appropriate setting of the
-REF_PATH environment variable.  For example, if seq_cache_populate was run
+REF_PATH environment varaiable.  For example, if seq_cache_populate was run
 using options '-root /tmp/ref_cache -subdirs 2', setting REF_PATH to
 '/tmp/ref_cache/%2s/%2s/%s' should allow samtools to find the references that
 it stored.
